 
#include "mvOs.h"
#include "mvBoardEnvLib.h"
#include "mvCtrlEnvLib.h"
#include "mvUsbDevApi.h"
#include "mvUsbDevPrv.h"
#include "mvUsbCh9.h"
#include "mvUsb.h"
#include "mvUsbRegs.h"
 
#include <linux/module.h> 
#include <linux/moduleparam.h>
#include <linux/usb_ch9.h>
#include <linux/usb_gadget.h>

#include <asm/irq.h>


#define DRIVER_VERSION	"05-Oct-2004"
#define	DRIVER_DESC		"Marvell Gadget USB Peripheral Controller"

struct mv_usb_dev;

struct mv_usb_ep 
{
   	struct usb_ep			ep;
    struct mv_usb_dev*		usb_dev;
    struct list_head        req_list;
    unsigned				num : 8,
                            is_enabled : 1,
						    is_in : 1;
};

struct mv_usb_dev 
{
	/* each pci device provides one gadget, several endpoints */
	struct usb_gadget			gadget;
	spinlock_t					lock;
	struct usb_gadget_driver 	*driver;
    struct mv_usb_ep			ep[2*ARC_USB_MAX_ENDPOINTS];
	unsigned					enabled : 1,
								protocol_stall : 1,
								got_irq : 1;
	u16							chiprev;
	struct device				*dev; 
	void*			            mv_usb_handle;
    int                         dev_no;
};


extern void   _usb_dci_vusb20_isr(pointer);

static void* mvUsbMalloc(unsigned int size)
{
	return kmalloc(size,GFP_ATOMIC);
}

static void mvUsbFree(void* buf)
{
	return kfree(buf);
}

static void* mvUsbIoUncachedMalloc( void* pDev, MV_U32 size, MV_ULONG* pPhyAddr )
{
    return pci_alloc_consistent( pDev, size, (dma_addr_t *)pPhyAddr );
}

static void mvUsbIoUncachedFree( void* pDev, MV_U32 size, MV_ULONG phyAddr, void* pVirtAddr )
{
    return pci_free_consistent( pDev, size, pVirtAddr, (dma_addr_t)phyAddr );
} 

static MV_ULONG mvUsbCacheInvalidate( void* pDev, void* p, int size )
{
    return pci_map_single( pDev, p, size, PCI_DMA_FROMDEVICE );
}

static MV_ULONG mvUsbCacheFlush( void* pDev, void* p, int size )
{
    return pci_map_single( pDev, p, size, PCI_DMA_TODEVICE );
}

static MV_U32  mvUsbBaseRegAddr(int devNo)
{
    return (INTER_REGS_BASE | MV_USB_CORE_CAP_LENGTH_REG(devNo));
}


USB_IMPORT_FUNCS    usbImportFuncs =
{
    mvOsPrintf,
    mvOsSPrintf,
    mvUsbIoUncachedMalloc,
    mvUsbIoUncachedFree,
    mvUsbMalloc,
    mvUsbFree,
    memset,
    memcpy,
    mvUsbCacheFlush,
    mvUsbCacheInvalidate,
    NULL,
    NULL,
    NULL,
    mvUsbBaseRegAddr,
    NULL/*mvUsbAutoCalibration*/
};


static struct mv_usb_dev*   the_controllers[MV_BOARD_MAX_USB_IF] = {NULL, NULL};

static const char driver_name [] = "mv_88fxx81_gadget";
static const char driver_desc [] = DRIVER_DESC;

static char ep_name [2*ARC_USB_MAX_ENDPOINTS][10] = 
{
	"ep0out", "ep0in",
};

static SETUP_STRUCT     mv_ctrl_req;

static struct usb_ep_ops mv_usb_ep_ops;

static uint_8 mv_usb_start_ep0(struct mv_usb_dev *mv_dev)
{
    ARC_DEBUG_TRACE(ARC_DEBUG_FLAG_INIT, "%s: mv_dev=%p, mv_usb_handle=%p, mv_ep=%p, usb_ep=%p\n", 
                __FUNCTION__, mv_dev, mv_dev->mv_usb_handle, &mv_dev->ep[0], &mv_dev->ep[0].ep);

    /* Init ep0 IN and OUT ???? */
    mv_dev->ep[0].is_enabled = 1;

    _usb_device_init_endpoint(mv_dev->mv_usb_handle, 0, mv_dev->ep[0].ep.maxpacket, 
                                ARC_USB_SEND,  ARC_USB_CONTROL_ENDPOINT, 0);

    _usb_device_init_endpoint(mv_dev->mv_usb_handle, 0, mv_dev->ep[0].ep.maxpacket, 
                                ARC_USB_RECV, ARC_USB_CONTROL_ENDPOINT, 0);

    return USB_OK;
}

static uint_8 mv_usb_reinit (struct mv_usb_dev *usb_dev)
{
    int                 i, num;
    MV_BOOL             is_in;
    struct mv_usb_ep	*ep;

    ARC_DEBUG_TRACE(ARC_DEBUG_FLAG_INIT, "%s: mv_dev=%p, mv_usb_handle=%p\n", 
                __FUNCTION__, usb_dev, usb_dev->mv_usb_handle);

   	INIT_LIST_HEAD (&usb_dev->gadget.ep_list);
    num = 0;
    for(i=0; i<2*ARC_USB_MAX_ENDPOINTS; i++)
    {
        is_in = i % 2;
        ep = &usb_dev->ep[i];
   		if (num != 0)
        {
			list_add_tail (&ep->ep.ep_list, &usb_dev->gadget.ep_list);
            sprintf(&ep_name[i][0], "ep%d%s", num, is_in ? "in" : "out");
        }

		ep->ep.name = &ep_name[i][0];
		ep->usb_dev = usb_dev;
		ep->num = num;
        ep->is_in = is_in;
        ep->is_enabled = 0;

        INIT_LIST_HEAD (&ep->req_list);
    
	    ep->ep.maxpacket = ~0;
	    ep->ep.ops = &mv_usb_ep_ops;
        if(is_in)
            num++;
    }
    usb_dev->ep[0].ep.maxpacket = 64;
   	usb_dev->gadget.ep0 = &usb_dev->ep[0].ep;
	INIT_LIST_HEAD (&usb_dev->gadget.ep0->ep_list);
    return USB_OK;
}

void mv_usb_bus_reset_service(void*      handle, 
                               uint_8     type, 
                               boolean    setup,
                               uint_8     direction, 
                               uint_8_ptr buffer,
                               uint_32    length, 
                               uint_8     error)
{
    int                     i, dev_no = _usb_device_get_dev_num(handle);
    struct mv_usb_dev	    *mv_dev = the_controllers[dev_no];
    struct mv_usb_ep        *mv_ep;
    struct usb_request*     usb_req;

    ARC_DEBUG_TRACE(ARC_DEBUG_FLAG_RESET, "%s\n", __FUNCTION__);

    /* Stop Hardware and cancel all pending requests */
    for (i=0; i<2*(ARC_USB_MAX_ENDPOINTS); i++)
    {
        mv_ep = &mv_dev->ep[i];
        while (!list_empty (&mv_ep->req_list)) 
        {
		    usb_req = list_entry (mv_ep->req_list.next, struct usb_request, list);

            /* Cancel transfer */
            _usb_device_cancel_transfer(handle, mv_ep->num, 
                            mv_ep->is_in ? ARC_USB_SEND : ARC_USB_RECV);

            /* Dequeue request and call complete function */
      	    list_del_init (&usb_req->list);

	        if (usb_req->status == -EINPROGRESS)
		        usb_req->status = -ESHUTDOWN;

            /* ????? what about enable interrupts */
            /*
            mvOsPrintf("%s ep_%d complete: status=%d\n", 
                            __FUNCTION__, i, usb_req->status);
            */
      	    usb_req->complete (&mv_ep->ep, usb_req);
        }
    }

    /* If connected call Function disconnect callback */
    if( (mv_dev->gadget.speed != USB_SPEED_UNKNOWN) && 
        (mv_dev->driver != NULL) &&
        (mv_dev->driver->disconnect != NULL) )

    {
        mv_dev->driver->disconnect (&mv_dev->gadget);
    }

    /* Reinit all endpoints */
    mv_usb_reinit(mv_dev);

    /* Restart Control Endpoint #0 */
    mv_usb_start_ep0(mv_dev);
}


void mv_usb_speed_service(void*      handle, 
                           uint_8     type, 
                           boolean    setup,
                           uint_8     direction, 
                           uint_8_ptr buffer,
                           uint_32    length, 
                           uint_8     error)
{
    int                     dev_no = _usb_device_get_dev_num(handle);
    struct mv_usb_dev	    *mv_dev = the_controllers[dev_no];

    ARC_DEBUG_TRACE(ARC_DEBUG_FLAG_SPEED, "%s: speed=%d\n", 
                            __FUNCTION__, (int)length);

    if(length == ARC_USB_SPEED_HIGH)
        mv_dev->gadget.speed = USB_SPEED_HIGH;
    else
        mv_dev->gadget.speed = USB_SPEED_FULL;

    return;
}

void mv_usb_suspend_service(void*      handle, 
                            uint_8     type, 
                            boolean    setup,
                            uint_8     direction, 
                            uint_8_ptr buffer,
                            uint_32    length, 
                            uint_8     error)
{
    int                     dev_no = _usb_device_get_dev_num(handle);
    struct mv_usb_dev	    *mv_dev = the_controllers[dev_no];

    ARC_DEBUG_TRACE(ARC_DEBUG_FLAG_SUSPEND, "%s\n", __FUNCTION__);

	if( (mv_dev->driver != NULL) &&
        (mv_dev->driver->suspend != NULL) )
	    mv_dev->driver->suspend (&mv_dev->gadget);
}

void mv_usb_resume_service(void*      handle, 
                            uint_8     type, 
                            boolean    setup,
                            uint_8     direction, 
                            uint_8_ptr buffer,
                            uint_32    length, 
                            uint_8     error)
{
    int                     dev_no = _usb_device_get_dev_num(handle);
    struct mv_usb_dev	    *mv_dev = the_controllers[dev_no];

    ARC_DEBUG_TRACE(ARC_DEBUG_FLAG_RESUME, "%s\n", __FUNCTION__);

	if( (mv_dev->driver != NULL) &&
        (mv_dev->driver->resume != NULL) )
	    mv_dev->driver->resume (&mv_dev->gadget);
}

void mv_usb_tr_complete_service(void*      handle, 
                                 uint_8     type, 
                                 boolean    setup,
                                 uint_8     direction, 
                                 uint_8_ptr buffer,
                                 uint_32    length, 
                                 uint_8     error)
{
    int                     dev_no = _usb_device_get_dev_num(handle);
    struct mv_usb_dev	    *mv_dev = the_controllers[dev_no];
    struct mv_usb_ep       *mv_ep;
    struct usb_request      *usb_req;
    int                     ep_num = (type*2) + direction;

    ARC_DEBUG_TRACE(ARC_DEBUG_FLAG_EP1, 
            "%s: ep_num=%d, setup=%s, direction=%s, pBuf=0x%x, length=%d, error=0x%x\n", 
             __FUNCTION__, type, setup ? "YES" : "NO", 
             (direction == ARC_USB_RECV) ? "RECV" : "SEND", 
             (unsigned)buffer, (int)length, error);

    mv_ep = &mv_dev->ep[ep_num];
    if( !list_empty(&mv_ep->req_list) )
    {
        usb_req = list_entry (mv_ep->req_list.next, struct usb_request, list);
        if(usb_req->buf != buffer)
        {
            mvOsPrintf("%s: ep=%d-%s, Unexpected buffer pointer: %p, len=%d, expected=%p\n", 
                    __FUNCTION__, ep_num, (direction == ARC_USB_RECV) ? "out" : "in",
                    buffer, length, usb_req->buf);
            return;       
        }
        /* Dequeue request and call complete function */
        list_del_init (&usb_req->list);

        /* ????? what about enable interrupts */
        usb_req->actual = length;
        usb_req->status = error;
        usb_req->complete (&mv_ep->ep, usb_req);
    }
    else
        printk("%s: ep=%p, epName=%s, epNum=%d - reqList EMPTY\n", 
                __FUNCTION__, mv_ep, mv_ep->ep.name, mv_ep->num);
}

void mv_usb_ep0_complete_service(void*      handle, 
                                 uint_8     type, 
                                 boolean    setup,
                                 uint_8     direction, 
                                 uint_8_ptr buffer,
                                 uint_32    length, 
                                 uint_8     error)
{ /* Body */
    int                     dev_no = _usb_device_get_dev_num(handle);
    struct mv_usb_dev	    *mv_dev = the_controllers[dev_no];
    struct mv_usb_ep       *mv_ep;
    struct usb_request*     usb_req;
    int                     rc;
   
    ARC_DEBUG_TRACE(ARC_DEBUG_FLAG_EP0, 
                "%s: EP0(%d), setup=%s, direction=%s, pBuf=0x%x, length=%d, error=0x%x\n", 
                __FUNCTION__, type, setup ? "YES" : "NO", 
                (direction == ARC_USB_RECV) ? "RECV" : "SEND", 
                (unsigned)buffer, (int)length, error);

    mv_ep = &mv_dev->ep[type];

    if (setup) 
    {
        _usb_device_read_setup_data(handle, type, (u8 *)&mv_ctrl_req);
        le16_to_cpus (&mv_ctrl_req.VALUE);
	    le16_to_cpus (&mv_ctrl_req.INDEX);
		le16_to_cpus (&mv_ctrl_req.LENGTH);

	    /* make sure any leftover request state is cleared */
        while (!list_empty (&mv_ep->req_list)) 
        {
		    usb_req = list_entry (mv_ep->req_list.next, struct usb_request, list);

            /* Cancel transfer */
            _usb_device_cancel_transfer(handle, mv_ep->num, direction);

            /* Dequeue request and call complete function */
      	    list_del_init (&usb_req->list);

	        if (usb_req->status == -EINPROGRESS)
		        usb_req->status = -EPROTO;

            /* ????? what about enable interrupts */
      	    usb_req->complete (&mv_ep->ep, usb_req);
        }
    }
    /* Setup request direction */
   	mv_ep->is_in = (mv_ctrl_req.REQUESTTYPE & REQ_DIR_IN) != 0;		

    if(setup)
        ARC_DEBUG_TRACE(ARC_DEBUG_FLAG_SETUP, 
                "Setup: reqType=0x%x, req=0x%x, value=0x%02x, index=0x%02x, length=0x%02x\n", 
                mv_ctrl_req.REQUESTTYPE, mv_ctrl_req.REQUEST, mv_ctrl_req.VALUE,
                mv_ctrl_req.INDEX, mv_ctrl_req.LENGTH);

    /* Handle most lowlevel requests;
	 * everything else goes uplevel to the gadget code.
	 */
	switch (mv_ctrl_req.REQUEST) 
    {
	    case REQ_GET_STATUS: 
            mvUsbCh9GetStatus(handle, setup, &mv_ctrl_req);
            break;

        case REQ_CLEAR_FEATURE:
            mvUsbCh9ClearFeature(handle, setup, &mv_ctrl_req);
            break;

        case REQ_SET_FEATURE:
            mvUsbCh9SetFeature(handle, setup, &mv_ctrl_req);
            break;

        case REQ_SET_ADDRESS:
            mvUsbCh9SetAddress(handle, setup, &mv_ctrl_req);
            break;

        default:
            /* All others delegate call up-layer gadget code */
            if(setup)
            {
                rc = mv_dev->driver->setup (&mv_dev->gadget, (struct usb_ctrlrequest*)&mv_ctrl_req);
                if(rc < 0)
                {
                    mvOsPrintf("%s: Setup is failed: req=0x%02x, reqType=0x%x, rc=%d\n", 
                           __FUNCTION__, mv_ctrl_req.REQUEST, mv_ctrl_req.REQUESTTYPE, rc);
                    _usb_device_stall_endpoint(handle, 0, ARC_USB_RECV);
                    return;
                }
                /* Akcnoledge  */
                if(mv_ep->is_in)
                    _usb_device_recv_data(handle, 0, NULL, 0);
            }
            break;
    }
    if(!setup)
    {
        if( !list_empty(&mv_ep->req_list) )
        {
            usb_req = list_entry (mv_ep->req_list.next, struct usb_request, list);

            /* Dequeue request and call complete function */
      	    list_del_init (&usb_req->list);

            usb_req->actual = length;
            usb_req->status = error;
            usb_req->complete (&mv_ep->ep, usb_req);
        }
    }
}

static irqreturn_t mv_usb_dev_irq (int irq, void *_dev, struct pt_regs* regs)
{
	struct mv_usb_dev		*mv_dev = _dev;

	spin_lock (&mv_dev->lock);

	/* handle ARC USB Device interrupts */
    _usb_dci_vusb20_isr(mv_dev->mv_usb_handle);

	spin_unlock (&mv_dev->lock);

	return IRQ_HANDLED;
}

/* when a driver is successfully registered, it will receive
 * control requests including set_configuration(), which enables
 * non-control requests.  then usb traffic follows until a
 * disconnect is reported.  then a host may connect again, or
 * the driver might get unbound.
 */
int usb_gadget_register_driver (struct usb_gadget_driver *driver)
{
	int					retval, dev_no, i;
	struct mv_usb_dev	*mv_dev = NULL;
    uint_8              error;

    /* Find USB Gadget device controller */
    for(dev_no=0; dev_no<mvCtrlUsbMaxGet(); dev_no++)
    {
        mv_dev = the_controllers[dev_no];
        if(mv_dev != NULL)
            break;
    }

	mvOsPrintf("ENTER usb_gadget_register_driver dev=%d, pDriver=%p\n", 
                dev_no, driver);

	if ( (driver == NULL)
			|| (driver->speed != USB_SPEED_HIGH)
			|| !driver->bind
			|| !driver->unbind
			|| !driver->setup)
		return -EINVAL;

	if (!mv_dev)
		return -ENODEV;

	if (mv_dev->driver)
		return -EBUSY;


	/* first hook up the driver ... */
	mv_dev->driver = driver;
	mv_dev->gadget.dev.driver = &driver->driver;
	retval = driver->bind (&mv_dev->gadget);
	if (retval) {
		mvOsPrintf("bind to driver %s --> %d\n",
				driver->driver.name, retval);
		mv_dev->driver = 0;
		mv_dev->gadget.dev.driver = 0;
		return retval;
	}

    /* First of all. */
    _usb_device_set_bsp_funcs(&usbImportFuncs);

    _usb_debug_set_flags(   ARC_DEBUG_FLAG_ERROR
                          | ARC_DEBUG_FLAG_STATS
                          /*
                          | ARC_DEBUG_FLAG_ADDR
                          | ARC_DEBUG_FLAG_SETUP
                          | ARC_DEBUG_FLAG_RESET
                          */
                          /*| ARC_DEBUG_FLAG_EP1 */
                          /*| ARC_DEBUG_FLAG_ISR*/
                          /*| ARC_DEBUG_FLAG_RX*/
                          /*| ARC_DEBUG_FLAG_TX*/
                          );

    /* Enable ARC USB device */
    retval = (int)_usb_device_init(dev_no, &mv_dev->mv_usb_handle);
    if (retval != USB_OK) 
    {
        mvOsPrintf("\nUSB Initialization failed. Error: %x", retval);
        return -EINVAL;
    } /* Endif */

    /* Register all ARC Services */  
    error = _usb_device_register_service(mv_dev->mv_usb_handle, 
                                ARC_USB_SERVICE_BUS_RESET, mv_usb_bus_reset_service);
    if (error != USB_OK) 
    {
        mvOsPrintf("\nUSB BUS_RESET Service Registration failed. Error: 0x%x", error);
        return -EINVAL;
    } /* Endif */
   
    error = _usb_device_register_service(mv_dev->mv_usb_handle, 
                        ARC_USB_SERVICE_SPEED_DETECTION, mv_usb_speed_service);
    if (error != USB_OK) 
    {
        mvOsPrintf("\nUSB SPEED_DETECTION Service Registration failed. Error: 0x%x", 
                        error);
        return -EINVAL;
    } /* Endif */
         
    error = _usb_device_register_service(mv_dev->mv_usb_handle, 
                                ARC_USB_SERVICE_SUSPEND, mv_usb_suspend_service);
    if (error != USB_OK) 
    {
        mvOsPrintf("\nUSB SUSPEND Service Registration failed. Error: 0x%x", error);
        return -EINVAL;
    } /* Endif */

    error = _usb_device_register_service(mv_dev->mv_usb_handle, 
                                ARC_USB_SERVICE_RESUME, mv_usb_resume_service);
    if (error != USB_OK) 
    {
        mvOsPrintf("\nUSB RESUME Service Registration failed. Error: 0x%x", error);
        return -EINVAL;
    } /* Endif */

    error = _usb_device_register_service(mv_dev->mv_usb_handle, 0, 
                                            mv_usb_ep0_complete_service);   
    if (error != USB_OK) 
    {
        mvOsPrintf("\nUSB ep0 TR_COMPLETE Service Registration failed. Error: 0x%x", error);
        return error;
    } /* Endif */


    error = mv_usb_start_ep0(mv_dev);

    /* request_irq */
	if (request_irq (IRQ_USB_CTRL(dev_no), mv_usb_dev_irq, SA_INTERRUPT, 
                     driver_name, mv_dev) != 0) 
    {
		printk("%s: request interrupt %d failed\n", __FUNCTION__, IRQ_USB_CTRL(dev_no));
		return -EBUSY;
    }

    for (i=1; i<ARC_USB_MAX_ENDPOINTS; i++)
    {
        error = _usb_device_register_service(mv_dev->mv_usb_handle, i, 
                                                    mv_usb_tr_complete_service);   
        if (error != USB_OK) 
        {
            mvOsPrintf("\nUSB ep0 TR_COMPLETE Service Registration failed. Error: 0x%x", error);
            return error;
        } /* Endif */
    }

	mvOsPrintf("registered USB gadget driver %s\n", driver->driver.name);
	return 0;
}
EXPORT_SYMBOL (usb_gadget_register_driver);

int usb_gadget_unregister_driver (struct usb_gadget_driver *driver)
{
    int                 i, dev_no;
    struct mv_usb_ep    *mv_ep;
    struct usb_request  *usb_req;
    struct mv_usb_dev	*mv_dev = NULL;

    /* Find USB Gadget device controller */
    for(dev_no=0; dev_no<mvCtrlUsbMaxGet(); dev_no++)
    {
        if( (the_controllers[dev_no] != NULL) && (the_controllers[dev_no]->driver == driver) )
        {
            mv_dev = the_controllers[dev_no];
            break;
        }
    }
    

	if (!mv_dev)
		return -ENODEV;
	if (!driver || (driver != mv_dev->driver) )
		return -EINVAL;

    /* Stop and Disable ARC USB device */

    /* Stop Endpoints */
    for (i=0; i<2*(ARC_USB_MAX_ENDPOINTS); i++)
    {
        mv_ep = &mv_dev->ep[i];
        if(mv_ep->is_enabled == 0)
            continue;

        while (!list_empty (&mv_ep->req_list)) 
        {
		    usb_req = list_entry (mv_ep->req_list.next, struct usb_request, list);

            /* Cancel transfer */
            _usb_device_cancel_transfer(mv_dev->mv_usb_handle, mv_ep->num, 
                            mv_ep->is_in ? ARC_USB_SEND : ARC_USB_RECV);

            /* Dequeue request and call complete function */
      	    list_del_init (&usb_req->list);

	        if (usb_req->status == -EINPROGRESS)
		        usb_req->status = -ESHUTDOWN;

      	    usb_req->complete (&mv_ep->ep, usb_req);
        }
        mv_ep->is_enabled = 0;
        _usb_device_deinit_endpoint(mv_dev->mv_usb_handle, mv_ep->num, 
                                mv_ep->is_in ? ARC_USB_SEND : ARC_USB_RECV);
    }
    for (i=0; i<ARC_USB_MAX_ENDPOINTS; i++)
        _usb_device_unregister_service(mv_dev->mv_usb_handle, i);   

    /* Deregister all other services */
    _usb_device_unregister_service(mv_dev->mv_usb_handle, ARC_USB_SERVICE_BUS_RESET);   
    _usb_device_unregister_service(mv_dev->mv_usb_handle, ARC_USB_SERVICE_SPEED_DETECTION);

    _usb_device_unregister_service(mv_dev->mv_usb_handle, ARC_USB_SERVICE_SUSPEND);

    _usb_device_unregister_service(mv_dev->mv_usb_handle, ARC_USB_SERVICE_RESUME);

    _usb_device_shutdown(mv_dev->mv_usb_handle);

    /* free_irq */
	free_irq (IRQ_USB_CTRL(dev_no), mv_dev);

    driver->unbind (&mv_dev->gadget);
	mv_dev->gadget.dev.driver = 0;
	mv_dev->driver = 0;

    /*device_remove_file(dev->dev, &dev_attr_function); ?????*/
	mvOsPrintf("unregistered Marvell USB %d gadget driver %s\n", dev_no, driver->driver.name);

    return 0;
}
EXPORT_SYMBOL (usb_gadget_unregister_driver);

void    mv_usb_show(struct mv_usb_dev* mv_dev, unsigned int mode)
{
    int     i;

    if( MV_BIT_CHECK(mode, 0) )
        _usb_regs(mv_dev->mv_usb_handle);

    if( MV_BIT_CHECK(mode, 1) )
        _usb_status(mv_dev->mv_usb_handle);

    if( MV_BIT_CHECK(mode, 2) )
        _usb_stats(mv_dev->mv_usb_handle);

    if( MV_BIT_CHECK(mode, 3) )
        _usb_debug_print_trace_log();

    for(i=0; i<ARC_USB_MAX_ENDPOINTS; i++)
    {
        if( MV_BIT_CHECK(mode, (8+i)) )
        {
            _usb_ep_status(mv_dev->mv_usb_handle, i, ARC_USB_RECV);
            _usb_ep_status(mv_dev->mv_usb_handle, i, ARC_USB_SEND);
        }
    }
}
EXPORT_SYMBOL (mv_usb_show);

static int  mv_usb_ep_enable(struct usb_ep *_ep, 
                              const struct usb_endpoint_descriptor *desc)
{
    struct mv_usb_dev* usb_dev;
	struct mv_usb_ep*  usb_ep;
    uint_16             maxSize;
    uint_8              epType; 
    unsigned long		flags;

    usb_ep = container_of (_ep, struct mv_usb_ep, ep);
	if( (_ep == NULL) || (desc == NULL) )
		return -EINVAL;
    /*
    mvOsPrintf("%s: ep=0x%x, num=%d, direction=%s, name=%s, desc=0x%x\n", 
                __FUNCTION__, (unsigned)_ep, usb_ep->num, 
                usb_ep->is_in ? "IN" : "OUT", _ep->name, (unsigned)desc);
    */
    if(usb_ep->is_enabled == 1)
    {
        mvOsPrintf("mv_usb: Endpoint is already in use\n");
	    return -EINVAL;
    }

    /*
    mvOsPrintf("\t bDescType=%d, maxPktSize=%d, bmAttr=%d, bEndAddr=%d\n\n",
                desc->bDescriptorType, desc->wMaxPacketSize, 
                desc->bmAttributes, desc->bEndpointAddress);
    */
    if(usb_ep->num == 0)
    {
        mvOsPrintf("mv_usb: ep0 is reserved\n");
	    return -EINVAL;
    }

    if(desc->bDescriptorType != USB_DT_ENDPOINT)
    {
        mvOsPrintf("mv_usb: wrong descriptor type %d\n", desc->bDescriptorType);
	    return -EINVAL;
    }

	usb_dev = usb_ep->usb_dev;
	if( (usb_dev->driver == NULL) || 
        (usb_dev->gadget.speed == USB_SPEED_UNKNOWN) )
    {
		return -ESHUTDOWN;
    }
    /* Max packet size */
    maxSize = le16_to_cpu (desc->wMaxPacketSize);

    /* Endpoint type */
    if( (desc->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK) == USB_ENDPOINT_XFER_CONTROL)
        epType = ARC_USB_CONTROL_ENDPOINT;
    else if( (desc->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK) == USB_ENDPOINT_XFER_ISOC) 
        epType = ARC_USB_ISOCHRONOUS_ENDPOINT;
    else if( (desc->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK) == USB_ENDPOINT_XFER_BULK) 
        epType = ARC_USB_BULK_ENDPOINT;
    else
        epType = ARC_USB_INTERRUPT_ENDPOINT;

   	spin_lock_irqsave (&usb_dev->lock, flags);
	_ep->maxpacket = maxSize & 0x7ff;
	usb_ep->is_enabled = 1;

    _usb_device_init_endpoint(usb_dev->mv_usb_handle, usb_ep->num, maxSize, 
            usb_ep->is_in ? ARC_USB_SEND : ARC_USB_RECV, epType,
            (epType == ARC_USB_BULK_ENDPOINT) ? ARC_USB_DEVICE_DONT_ZERO_TERMINATE : 0);

    spin_unlock_irqrestore (&usb_dev->lock, flags);
    return 0;
}

static int  mv_usb_ep_disable (struct usb_ep *_ep)
{
    struct mv_usb_dev* usb_dev;
	struct mv_usb_ep*  usb_ep;
    struct usb_request* usb_req;
    unsigned long		flags;
    int                 count = 0;
    uint_8              direction;

    usb_ep = container_of (_ep, struct mv_usb_ep, ep);
	if( (_ep == NULL) || (usb_ep->is_enabled == 0) || (usb_ep->num == 0))
		return -EINVAL;

    usb_dev = usb_ep->usb_dev;

    /*
    mvOsPrintf("mv_usb_ep_disable: ep=0x%x, num=%d, direction=%s, name=%s\n", 
                (unsigned)_ep, usb_ep->num, usb_ep->is_in ? "IN" : "OUT", 
                _ep->name);
    */
   	spin_lock_irqsave (&usb_dev->lock, flags);

    direction = usb_ep->is_in ? ARC_USB_SEND : ARC_USB_RECV;
    /* Cancell all requests ???? */
    while (!list_empty (&usb_ep->req_list)) 
    {
		usb_req = list_entry (usb_ep->req_list.next, struct usb_request, list);

        /* Cancel transfer */
        _usb_device_cancel_transfer(usb_dev->mv_usb_handle, usb_ep->num, direction);

        /* Dequeue request and call complete function */
      	list_del_init (&usb_req->list);

	    if (usb_req->status == -EINPROGRESS)
		    usb_req->status = -ESHUTDOWN;

        /* ????? what about enable interrupts */
        /*mvOsPrintf("%s complete: status=%d\n", __FUNCTION__, usb_req->status);*/
      	usb_req->complete (&usb_ep->ep, usb_req);

        count++;
    }
    if(count > 0)
        mvOsPrintf("\t %d transfers are cancelled\n\n", count);

    /* Disable endpoint */
    _usb_device_deinit_endpoint(usb_dev->mv_usb_handle, usb_ep->num, direction);

    spin_unlock_irqrestore (&usb_dev->lock, flags);
    return 0;
}


static struct usb_request* mv_usb_ep_alloc_request(struct usb_ep *_ep, int gfp_flags)
{
	struct usb_request* req;

	if (!_ep)
		return NULL;

	req = kmalloc (sizeof *req, gfp_flags);
	if (!req)
		return NULL;

	memset (req, 0, sizeof *req);
	INIT_LIST_HEAD (&req->list);

    return req;
}

static void     mv_usb_ep_free_request(struct usb_ep *_ep, struct usb_request *_req)
{

	if (!_ep || !_req)
    {
        mvOsPrintf("mv_usb_ep_free_request: wrong parameters\n");
		return;
    }
	if( !list_empty(&_req->list) )
    {
        mvOsPrintf("mv_usb_ep_free_request: _req=0x%x - req_list is not empty\n",
                    (unsigned)_req);
    }
	kfree (_req);
}

static void*    mv_usb_ep_alloc_buffer(struct usb_ep* _ep, unsigned bytes,
	                                    dma_addr_t* dma, int gfp_flags)
{
   	void			    *buf_ptr;

	if (!_ep)
		return NULL;

	buf_ptr = kmalloc(bytes, gfp_flags);
	if (buf_ptr)
		*dma = virt_to_phys(buf_ptr);

    return buf_ptr;
}

static void     mv_usb_ep_free_buffer(struct usb_ep *_ep, void *buf,
	                                   dma_addr_t dma, unsigned bytes) 
{
    kfree (buf);
}

static int      mv_usb_ep_queue (struct usb_ep *_ep, struct usb_request *_req, 
                                  int gfp_flags)
{
    struct mv_usb_dev* usb_dev;
	struct mv_usb_ep*  usb_ep;
    unsigned long       flags;
    uint_8              error;

    usb_ep = container_of (_ep, struct mv_usb_ep, ep);
    /* check parameters */
	if( (_ep == NULL) || (_req == NULL) || (usb_ep->is_enabled == 0) )
		return -EINVAL;

	usb_dev = usb_ep->usb_dev;

    ARC_DEBUG_TRACE(ARC_DEBUG_FLAG_RX | ARC_DEBUG_FLAG_TX, 
                "%s: ep=0x%x, mv_ep=%p, num=%d-%s, name=%s, driver=0x%x, speed=%d\n", 
                __FUNCTION__, (unsigned)_ep, usb_ep, usb_ep->num, 
                usb_ep->is_in ? "in" : "out", _ep->name, (unsigned)usb_dev->driver, 
                usb_dev->gadget.speed);

	if ( (usb_dev->driver == NULL) || (usb_dev->gadget.speed == USB_SPEED_UNKNOWN) )
		return -ESHUTDOWN;

  	spin_lock_irqsave (&usb_dev->lock, flags);

	_req->status = -EINPROGRESS;
	_req->actual = 0;

    /* Add request to list */
    if( ((usb_ep->num == 0) && (_req->length == 0)) || (usb_ep->is_in) )
    {
        error = _usb_device_send_data(usb_dev->mv_usb_handle, usb_ep->num, _req->buf, _req->length);
    }
    else
    {
        error = _usb_device_recv_data(usb_dev->mv_usb_handle, usb_ep->num, _req->buf, _req->length);
    }
    if(error != USB_OK)
    {
        mvOsPrintf("mv_usb_ep_queue: Can't %s data (err=%d): ep_num=%d, pBuf=0x%x, size=%d\n",
                        usb_ep->is_in ? "send" : "recv", error, usb_ep->num, 
                        (unsigned)_req->buf, _req->length);
    }
    else
        list_add_tail(&_req->list, &usb_ep->req_list);

	spin_unlock_irqrestore (&usb_dev->lock, flags);

    return (int)error;
}

/* Cancell request */
static int      mv_usb_ep_dequeue (struct usb_ep *_ep, struct usb_request *_req)
{
    struct mv_usb_dev* usb_dev;
    struct usb_request *usb_req;
	struct mv_usb_ep*  usb_ep;
    unsigned long       flags;
    int                 status = 0;

    usb_ep = container_of (_ep, struct mv_usb_ep, ep);
    /* check parameters */
	if( (_ep == NULL) || (_req == NULL) || (usb_ep->is_enabled == 0) )
		return -EINVAL;

	usb_dev = usb_ep->usb_dev;

    /*
    mvOsPrintf("mv_usb_ep_dequeue: ep=0x%x, num=%d-%s, name=%s, driver=0x%x, speed=%d\n", 
                (unsigned)_ep, usb_ep->num, usb_ep->is_in ? "in" : "out", 
                _ep->name, (unsigned)usb_dev->driver, usb_dev->gadget.speed);
    */
	if ( (usb_dev->driver == NULL) || (usb_dev->gadget.speed == USB_SPEED_UNKNOWN) )
		return -ESHUTDOWN;

  	spin_lock_irqsave (&usb_dev->lock, flags);

    /* ????? Currently supported only dequeue request from the HEAD of List */
    if( !list_empty(&usb_ep->req_list) )
    {
        usb_req = list_entry (usb_ep->req_list.next, struct usb_request, list);

        if(usb_req == _req)
        {
            /* Cancel transfer */
            _usb_device_cancel_transfer(usb_dev->mv_usb_handle, usb_ep->num, 
                            usb_ep->is_in ? ARC_USB_SEND : ARC_USB_RECV);
            /* Dequeue request and call complete function */
      	    list_del_init (&_req->list);

	        if (_req->status == -EINPROGRESS)
		        _req->status = -ECONNRESET;

            /* ????? what about enable interrupts */
            /*mvOsPrintf("%s complete: status=%d\n", __FUNCTION__, _req->status);*/
      	    _req->complete (&usb_ep->ep, _req);
        }
        else
        {
            mvOsPrintf("%s: Unsupported ep=%p, req=%p\n", __FUNCTION__, _ep, _req);
            status = EINVAL;
        }
    }
    /*
    else
        mvOsPrintf("%s: ep=%p, epName=%s, epNum=%d - reqList EMPTY\n", 
                    __FUNCTION__, usb_ep, usb_ep->ep.name, usb_ep->num);
    */
	spin_unlock_irqrestore (&usb_dev->lock, flags);

    return status;
}

static int      mv_usb_ep_set_halt (struct usb_ep *_ep, int value)
{
    struct mv_usb_ep*   mv_ep;
	unsigned long		flags;
    int			        retval = 0;

  	mv_ep = container_of (_ep, struct mv_usb_ep, ep);
	if (_ep == NULL)
		return -EINVAL;
	if( (mv_ep->usb_dev->driver == NULL) || 
        (mv_ep->usb_dev->gadget.speed == USB_SPEED_UNKNOWN) )
		return -ESHUTDOWN;

    mvOsPrintf("%s: ep=%p, ep_name=%s, value=%d\n", 
                __FUNCTION__, _ep, _ep->name, value);

  	spin_lock_irqsave (&mv_ep->usb_dev->lock, flags);
	if (!list_empty (&mv_ep->req_list))
		retval = -EAGAIN;
	else 
    {
		/* set/clear, then synch memory views with the device */
		if (value) 
        {
			if (mv_ep->num == 0)
				mv_ep->usb_dev->protocol_stall = 1;
			else
				_usb_device_stall_endpoint(mv_ep->usb_dev->mv_usb_handle, mv_ep->num,
                mv_ep->is_in ? ARC_USB_SEND : ARC_USB_RECV);
		} 
        else
        {
		    _usb_device_unstall_endpoint(mv_ep->usb_dev->mv_usb_handle, mv_ep->num, 
                                    mv_ep->is_in ? ARC_USB_SEND : ARC_USB_RECV);
        }
	}
	spin_unlock_irqrestore (&mv_ep->usb_dev->lock, flags);

    return retval;
}


static void     mv_usb_ep_fifo_flush (struct usb_ep *_ep)
{
    ARC_DEBUG_TRACE(ARC_DEBUG_FLAG_TRACE, 
        "%s: ep=%p, ep_name=%s - NOT supported\n", __FUNCTION__, _ep, _ep->name);
}


static struct usb_ep_ops mv_usb_ep_ops = 
{
	.enable		    = mv_usb_ep_enable,
	.disable	    = mv_usb_ep_disable,

	.alloc_request	= mv_usb_ep_alloc_request,
	.free_request	= mv_usb_ep_free_request,

	.alloc_buffer	= mv_usb_ep_alloc_buffer,
	.free_buffer	= mv_usb_ep_free_buffer,

	.queue		    = mv_usb_ep_queue,
	.dequeue	    = mv_usb_ep_dequeue,

	.set_halt	    = mv_usb_ep_set_halt,
	.fifo_flush	    = mv_usb_ep_fifo_flush,
	/*.fifo_status    =  Not supported */
};

static int mv_usb_get_frame (struct usb_gadget *_gadget)
{
    ARC_DEBUG_TRACE(ARC_DEBUG_FLAG_TRACE, "Call mv_usb_get_frame - NOT supported\n");
    return 0;
}

static int mv_usb_wakeup(struct usb_gadget *_gadget)
{
    ARC_DEBUG_TRACE(ARC_DEBUG_FLAG_TRACE, "Call mv_usb_wakeup - NOT supported\n");
    return 0;
}

static int mv_usb_set_selfpowered (struct usb_gadget *_gadget, int value)
{
    ARC_DEBUG_TRACE(ARC_DEBUG_FLAG_TRACE, "Call mv_usb_set_selfpowered - NOT supported\n");
    return 0;
}

static const struct usb_gadget_ops mv_usb_ops = 
{
	.get_frame	     = mv_usb_get_frame,
	.wakeup		     = mv_usb_wakeup,
	.set_selfpowered = mv_usb_set_selfpowered,
    .ioctl           = NULL,
};

static void mv_usb_gadget_release (struct device *_dev)
{
	struct mv_usb_dev	*usb_dev = dev_get_drvdata (_dev);

    /*mvOsPrintf("Call mv_usb_gadget_release \n");*/
	mvOsFree(usb_dev);
}


static int __init mv_usb_gadget_probe(struct device *_dev) 
{
	struct mv_usb_dev	    *usb_dev;
    int                     dev_no;
    struct platform_device  *pDev = to_platform_device(_dev); 

    for(dev_no=0; dev_no<mvCtrlUsbMaxGet(); dev_no++)
    {
        if( (pDev->resource[1].flags == IORESOURCE_IRQ) &&
            (pDev->resource[1].start == IRQ_USB_CTRL(dev_no)) )
        {
            break;
        }
    }
    if(dev_no >= mvCtrlUsbMaxGet())
    {
        printk("mv_gadget_probe: device is not found\n");
        return -EINVAL;
    }
	printk(KERN_INFO "USB-%d Gadget driver probed\n", dev_no);

	if (the_controllers[dev_no]) 
    {
		printk("mv_dev_load: USB-%d controller is BUSY\n", dev_no);
		return -EBUSY;
	}

	/* alloc, and start init */
	usb_dev = mvOsMalloc (sizeof(struct mv_usb_dev));
	if (usb_dev == NULL)
	{
		printk("mv_dev_load: malloc failed\n");
		return -ENOMEM;
	}

	memset (usb_dev, 0, sizeof *usb_dev);
	spin_lock_init (&usb_dev->lock);
	usb_dev->dev = _dev; 
	usb_dev->gadget.ops = &mv_usb_ops;
	usb_dev->gadget.is_dualspeed = 1;

	/* the "gadget" abstracts/virtualizes the controller */
	strcpy (usb_dev->gadget.dev.bus_id, "gadget");
	usb_dev->gadget.dev.parent = _dev;
	usb_dev->gadget.dev.dma_mask = _dev->dma_mask; /* ?????? */
	usb_dev->gadget.dev.release = mv_usb_gadget_release ;
	usb_dev->gadget.name = driver_name;
    usb_dev->mv_usb_handle = NULL;
    usb_dev->dev_no = dev_no;

	dev_set_drvdata(_dev, usb_dev);
	the_controllers[dev_no] = usb_dev;

	/* Reset ARC USB device ????? */
	/* Reinit ARC USB device ????? */
    if( mv_usb_reinit (usb_dev) != USB_OK)
        return -EINVAL;

	device_register (&usb_dev->gadget.dev);

	return 0; 
}

static int __exit mv_usb_gadget_remove(struct device *_dev)
{
	struct mv_usb_dev	*usb_dev = dev_get_drvdata(_dev); 

	/* start with the driver above us */
	if (usb_dev->driver) 
	{
		/* should have been done already by driver model core */
		mvOsPrintf("pci remove, driver '%s' is still registered\n",
				    usb_dev->driver->driver.name);
		usb_gadget_unregister_driver (usb_dev->driver);
	}

	the_controllers[usb_dev->dev_no] = 0;
	device_unregister (&usb_dev->gadget.dev);

	dev_set_drvdata(_dev, 0);
	return 0;
}
 

static struct device_driver mv_usb_gadget_driver = 
{
	.name		= (char *) driver_name,
	.bus		= &platform_bus_type,
	.probe		= mv_usb_gadget_probe,
	.remove		= __exit_p(mv_usb_gadget_remove), 
};

MODULE_DESCRIPTION (DRIVER_DESC);
MODULE_AUTHOR ("Dima Epshtein");
MODULE_LICENSE ("GPL");

static int __init init (void)
{
	printk(KERN_INFO "%s: version %s loaded\n", driver_name, DRIVER_VERSION);
	return driver_register(&mv_usb_gadget_driver); 
}
module_init (init);

static void __exit cleanup (void)
{
	printk(KERN_INFO "%s: version %s unloaded\n", driver_name, DRIVER_VERSION);
    driver_unregister(&mv_usb_gadget_driver);
}
module_exit (cleanup);
